//
// EditorMouseAdapter.java for Vcsn in /home/loulou/enst/VcsnInterface
//
// Made by Loulou Pouchet
// Login   <loulou@lrde.epita.fr>
//
// Started on  Wed Dec 22 19:39:30 2004 Loulou Pouchet
// Last update Thu Dec  7 20:30:04 2006 Guillaume
//

import vaucanson.vgitoolkit.*;


import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.Point;
import java.awt.geom.Point2D;

import java.util.ArrayList;

public class EditorMouseAdapter extends MouseAdapter
{
    public EditorMouseAdapter(Storage storage)
    {
	super();
	storage_ = storage;
	states_ = new ArrayList();
	delta_ = new ArrayList();
    }


    public void				mouseClicked(MouseEvent e)
    {
	int idx = storage_.tabbed_pane_get().getSelectedIndex();
	VJGraph graph = storage_.graph_get(idx);
	// Get Cell under Mousepointer
	Point2D p = e.getPoint();
	graph.fromScreen(p);

	int x = (int)Math.round(p.getX());
	int y = (int)Math.round(p.getY());
	Object cell = graph.getFirstCellForLocation(x, y);

	Point2D mp = new Point2D.Double((double)p.getX(), (double)p.getY());
	Point2D rp = GeometricUtilities.toOrigin(graph, mp);
	x = (int)Math.round(rp.getX());
	y = (int)Math.round(rp.getY());

	Debug.echo("Mouse event: clicked");
	Debug.echo(x);
	Debug.echo(y);

	if (e.getClickCount() == 1)
	{
	    switch (storage_.mode_get())
	    {
		case VJToolKitConstants.MODE_SELECT:
		    // Update properties windows.
		    if (cell != null)
			if (((cell instanceof State ||
			      cell instanceof TerminalTransition) &&
			     storage_.state_properties_get().is_visible)
			    || (cell instanceof Transition &&
				storage_.transition_properties_get()
				.is_visible))
			    EditorCommands.properties(storage_);
		    break;
		case VJToolKitConstants.MODE_CREATE:
		    EditorCommands.create_state_at(storage_, x, y);
		    storage_.mode_set(VJToolKitConstants.MODE_SELECT);
		    break;
		case VJToolKitConstants.MODE_ORDER:
		    EditorCommands.order_state_at(storage_, x, y);
		    break;
		default:
		    break;
	    }
	}
	else if (e.getClickCount() == 2)
	{
	    // Display properties windows.
	}
    }


    public void			mousePressed(MouseEvent e)
    {
	// Get graph
	int idx = storage_.tabbed_pane_get().getSelectedIndex();
	VJGraph graph = storage_.graph_get(idx);
	Point2D p = e.getPoint();
	graph.fromScreen(p);

	Point2D mp = new Point2D.Double((double)p.getX(), (double)p.getY());
	Point2D rp = GeometricUtilities.toOrigin(graph, mp);
	int x = (int)Math.round(rp.getX());
	int y = (int)Math.round(rp.getY());

	Debug.echo("Mouse event: pressed");
	Debug.echo(x);
	Debug.echo(y);

	// Get Cell under Mousepointer
	Object[] cells = graph.getSelectionCells();

	if (cells != null && storage_.mode_get() == VJToolKitConstants.MODE_SELECT)
	{
	    states_.clear();
	    delta_.clear();
	    for (int i = 0; i < cells.length; i++)
		if (cells[i] instanceof State)
		{
		    State s = (State) cells[i];
		    states_.add(s);
		    Point d = new Point();
		    d.setLocation(GeometryProperties.getX(s) - x,
				  GeometryProperties.getY(s) - y);
		    delta_.add(d);
		}
	    if (!states_.isEmpty())
		storage_.mode_set(VJToolKitConstants.MODE_DRAG);
	}
    }

    public void			mouseReleased(MouseEvent e)
    {
	// Get graph
	int idx = storage_.tabbed_pane_get().getSelectedIndex();
	VJGraph graph = storage_.graph_get(idx);

	Point2D p = e.getPoint();
	graph.fromScreen(p);

	Point2D mp = new Point2D.Double((double)p.getX(), (double)p.getY());
	Point2D rp = GeometricUtilities.toOrigin(graph, mp);
	int x = (int)Math.round(rp.getX());
	int y = (int)Math.round(rp.getY());

	Debug.echo("Mouse event: released");
	Debug.echo(x);
	Debug.echo(y);

	if (states_ != null && storage_.mode_get() == VJToolKitConstants.MODE_DRAG)
	{
	    for (int i = 0; i < states_.size(); ++i)
	    {
		State s = (State) states_.get(i);
		Point d = (Point) delta_.get(i);
		GeometryProperties.setX(s, x + (int)d.getX());
		GeometryProperties.setY(s, y + (int)d.getY());
	    }
	    storage_.mode_set(VJToolKitConstants.MODE_SELECT);
	}
    }


    // Attributes.
    protected Storage		storage_;
    protected ArrayList		states_;
    protected ArrayList		delta_;
}
