//
// IO.java for Vcsn in /home/loulou/enst/VcsnInterface
// 
// Made by Loulou Pouchet
// Login   <loulou@lrde.epita.fr>
// 
// Started on  Fri Dec 10 16:03:09 2004 Loulou Pouchet
// Last update Mon Oct  3 15:05:01 2005 Loulou Pouchet
//

import vaucanson.vgitoolkit.*;


import org.xml.sax.SAXException;
import java.io.IOException;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.OutputStream;
import java.io.FileInputStream;
import java.io.DataInputStream;
import java.lang.Process;
import java.lang.Runtime;


// Global class for I/O operations with XML automaton.
public class IO
{
    public static String	launch_rat_algorithm(String[] arg,
						     VJGraph out)
    {
	String res = call_noinput_program(arg);

	if (res != null && res != "")
	    {
		VJGraph graph = XMLReader.string_to_graph(res);
		VJTools.set_state_mapping(graph);
		out.setModel(graph.getModel());
		VJTools.set_state_mapping(out);
 		LayoutTemplate.apply_global_layout(out);
		out.set_automaton_structure(graph.automaton_structure_get());
	    }
	return res;
    }



    // Launch algorithms where input is only one automaton.
    // Beware of support of the algoritm by the program.
    public static String	launch_algorithm(String[] arg,
						 VJGraph in,
						 VJGraph out)
    {
	String res = call_program(arg, XMLWriter.graph_to_string(in));

	if (res != null && res != "")
	    {
		VJGraph graph = XMLReader.string_to_graph(res);
		VJTools.set_state_mapping(graph);
		out.setModel(graph.getModel());
		VJTools.set_state_mapping(out);
 		LayoutTemplate.apply_global_layout(out);
		out.set_automaton_structure(graph.automaton_structure_get());
	    }
	return res;
    }


    // Launch algorithms where input is only one automaton.
    // Beware of support of the algoritm by the program.
    public static String	launch_algorithm(String[] arg,
						 VJGraph in)
    {
	String res = call_program(arg, XMLWriter.graph_to_string(in));
	
	return res;
    }



    // Launch algorithms where input is two automata.
    // Beware of support of the algoritm by the program.
    public static String	launch_algorithm(String[] arg,
						 VJGraph in1,
						 VJGraph in2,
						 VJGraph out)
    {
	String res = "";
	String xml_in1 = XMLWriter.graph_to_string(in1);
	String xml_in2 = XMLWriter.graph_to_string(in2);
	int i;

	try
	    {
		FileOutputStream out_stream = 
		    new FileOutputStream(VJToolKitConstants.TMP_FILENAME_2);
		out_stream.write(xml_in1.getBytes());
		out_stream.close();
	    }
	catch (IOException e)
	    {
		System.err.println(e);
	    }

	String[] cmd = new String[arg.length + 1];
	for (i = 0; i < arg.length; ++i)
	    cmd[i] = arg[i];
	cmd[i] = VJToolKitConstants.TMP_FILENAME_2;

	res = call_program(cmd, XMLWriter.graph_to_string(in2));

	if (res != null && res != "")
	    {
		VJGraph graph = XMLReader.string_to_graph(res);
		VJTools.set_state_mapping(graph);
		out.setModel(graph.getModel());
		VJTools.set_state_mapping(out);
 		LayoutTemplate.apply_global_layout(out);
		out.set_automaton_structure(graph.automaton_structure_get());
	    }
	return res;
    }



    // Call a program. 
    // Parameters: 
    // command:String	->
    // in:String	-> String to put on standard input. 
    // Returns:
    // String		-> String containing standard output.	
    protected static String	call_program(String[] command, 
					     String in)
    {
	String			res = "", tmp, error = "";

	try 
	    {
		FileOutputStream out = 
		    new FileOutputStream(VJToolKitConstants.TMP_FILENAME_1);
		out.write(in.getBytes());
		out.close();

		String[] cmd = new String[command.length + 1];
		cmd[0] = command[0];
		cmd[1] = command[1];
		cmd[2] = VJToolKitConstants.TMP_FILENAME_1;
		for (int i = 2; i < command.length; ++i)
		    cmd[i + 1] = command[i];

 		Debug.echo(cmd);
//  		Debug.echo(in);

 		Process p = Runtime.getRuntime().exec(cmd);

		// Read output from program.
		DataInputStream in_stream = 
		    new DataInputStream(p.getInputStream());
		DataInputStream err_stream = 
		    new DataInputStream(p.getErrorStream());
		while ((tmp = in_stream.readLine()) != null) 
		    res += tmp + "\n";
		in_stream.close();

		while ((tmp = err_stream.readLine()) != null) 
		    error += tmp + "\n";
		err_stream.close();

		if (error != "")
		    Debug.echo(error);

		p.waitFor();
		if (p.exitValue() != 0)
		    Debug.echo("Program returned " + p.exitValue());

		return res;
	    }
	catch (FileNotFoundException e)
	    {
		System.err.println(e);
	    }
	catch (IOException e)
	    {
		System.err.println(e);
	    }
	catch (java.lang.InterruptedException e)
	    {
		System.err.println(e);
	    }
	return null;
    }



    // Call a program. 
    // Parameters: 
    // command:String	->
    // in:String	-> String to put on standard input. 
    // Returns:
    // String		-> String containing standard output.	
    protected static String	call_noinput_program(String[] cmd)
    {
	String			res = "", tmp, error = "";
	int			i;

	try 
	    {
 		Process p = Runtime.getRuntime().exec(cmd);

		// Read output from program.
		DataInputStream in_stream = 
		    new DataInputStream(p.getInputStream());
		DataInputStream err_stream = 
		    new DataInputStream(p.getErrorStream());
		while ((tmp = in_stream.readLine()) != null) 
		    res += tmp + "\n";
		in_stream.close();

		Debug.echo(cmd);

		while ((tmp = err_stream.readLine()) != null) 
		    error += tmp + "\n";
		err_stream.close();

		if (error != "")
		    Debug.echo(error);

		p.waitFor();
		if (p.exitValue() != 0)
		    Debug.echo("Program returned " + p.exitValue());

		return res;
	    }
	catch (FileNotFoundException e)
	    {
		System.err.println(e);
	    }
	catch (IOException e)
	    {
		System.err.println(e);
	    }
	catch (java.lang.InterruptedException e)
	    {
		System.err.println(e);
	    }
	return null;
    }



}
