//
// TransitionProperties.java for Vcsn in /home/loulou/enst/VcsnInterface
//
// Made by Loulou Pouchet
// Login   <loulou@lrde.epita.fr>
//
// Started on  Tue Dec 14 16:24:38 2004 Loulou Pouchet
// Last update Tue Jun 21 01:33:34 2005 Louis-Noel Pouchet
//

import vaucanson.vgitoolkit.*;



import java.awt.*;
import java.awt.event.*;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.ButtonGroup;
import javax.swing.JMenuBar;
import javax.swing.KeyStroke;
import javax.swing.ImageIcon;

import javax.swing.JPanel;
import javax.swing.JTextArea;
import javax.swing.JScrollPane;
import javax.swing.JFrame;

import javax.swing.filechooser.*;
import java.io.*;
import javax.swing.*;


import java.util.Map;
import java.util.Vector;
// JGraph
import org.jgraph.graph.GraphConstants;
import org.jgraph.graph.CellView;
import org.jgraph.graph.DefaultGraphModel;
import org.jgraph.JGraph;


public class TransitionProperties
    extends PropertiesTemplate
    implements ActionListener, ItemListener
{
    TransitionProperties(Storage storage)
    {
	super(storage);
	frame_.setTitle("Transition properties");
	storage_.transition_properties_set(this);
	create_frame_elements();
    }


    protected JPanel		create_transition_panel()
    {
	JLabel label1 = new JLabel("Source state");
	JLabel label2 = new JLabel("Destination state");
	JLabel label3 = new JLabel("Label");

	src_ = new JComboBox();
	dst_ = new JComboBox();
	label_ = new JTextField();

	JPanel panel = new JPanel(new GridLayout(3, 2));
	panel.add(src_);
	panel.add(label1);
	panel.add(dst_);
	panel.add(label2);
	panel.add(label_);
	panel.add(label3);

	return panel;
    }



    protected JPanel		create_advanced_panel()
    {
	JPanel panel = new JPanel(new GridLayout(4, 1));
	JPanel panel1 = new JPanel(new BorderLayout());
	JPanel panel2 = new JPanel(new BorderLayout());
	JPanel panel3 = new JPanel(new BorderLayout());
	JPanel panel4 = new JPanel(new BorderLayout());
	String[] cardinals = {"N", "S", "E", "W", "NE", "NW", "SE", "SW"};
	String[] loop_sizes = { "Normal", "Small"};
	String[] trans_types = { GeometryProperties.LINE, GeometryProperties.ARCL,
				 GeometryProperties.ARCR };
	loop_angle_ = new JComboBox(cardinals);
	panel1.add(loop_angle_, BorderLayout.WEST);
	panel1.add(new JLabel("Loop angle"), BorderLayout.EAST);

	loop_size_ = new JComboBox(loop_sizes);
	panel2.add(loop_size_, BorderLayout.WEST);
	panel2.add(new JLabel("Loop size"), BorderLayout.EAST);


	trans_type_ = new JComboBox(trans_types);
	panel4.add(trans_type_, BorderLayout.WEST);
	panel4.add(new JLabel("Transition type"), BorderLayout.EAST);

	label_posit_ = new JTextField();
	panel3.add(label_posit_, BorderLayout.WEST);
	panel3.add(new JLabel("Label position"), BorderLayout.EAST);

	panel.add(panel1);
	panel.add(panel2);
	panel.add(panel3);
	panel.add(panel4);

	return panel;
    }


    protected void		create_frame_elements()
    {
	JPanel normal_panel = new JPanel(new BorderLayout());
	normal_panel.add(create_transition_panel(), BorderLayout.NORTH);

	JPanel advanced_panel = new JPanel(new GridLayout(1, 1));
	advanced_panel.add(create_advanced_panel());
	frame_panels_set(normal_panel, advanced_panel);
    }


    public void			actionPerformed(ActionEvent e)
    {
 	Object source = e.getSource();

 	if (source == bok_)
	{
	    set_transition_properties(trans_);
	    is_changed_ = true;
	    frame_hide();
	}
	else if (source == bsave_)
	    set_transition_properties(trans_);
 	else if (source == bcancel_)
	{
	    is_changed_ = false;
	    frame_hide();
	}
    }


    public void			itemStateChanged(ItemEvent e)
    {

    }


    public void			frame_show(Transition trans)
    {
	get_transition_properties(trans);
	trans_ = trans;
	frame_show();
    }


    public void			set_transition_properties(Transition trans)
    {
	trans.label_set(label_.getText());

	try
	{
	    int d = Integer.parseInt(label_posit_.getText());
	    GeometryProperties.setLabelPosition(trans, d);
	}
	catch (java.lang.NumberFormatException e)
	{
	    label_posit_.setText(Integer.toString
				 (GeometryProperties.getLabelPosition(trans)));
	}

	trans.connect((State) src_.getSelectedItem(),
		      (State) dst_.getSelectedItem());
	GeometryProperties.setTransitionType(trans,
					     (String)trans_type_.getSelectedItem());

	if (trans.dst_get() == trans.src_get())
	{
	    set_loop_angle(trans);
	    set_loop_size(trans);
	}

 	// Refresh view.
	int idx = storage_.tabbed_pane_get().getSelectedIndex();
	Tools.refresh_view(storage_.graph_get(idx));
    }


    public void			get_transition_properties(Transition trans)
    {
	// Clear combos.
	src_.removeAllItems();
	dst_.removeAllItems();
	// Fill combo boxes with existing states.
	int idx = storage_.tabbed_pane_get().getSelectedIndex();
	State[] states = VJGraph.get_states(storage_.graph_get(idx));

	for (int i = 0; i < states.length; ++i)
	{
	    State s = states[i];
	    src_.addItem(s);
	    dst_.addItem(s);
	}


	src_.setSelectedItem(trans.src_get());
	dst_.setSelectedItem(trans.dst_get());
	String ttype = GeometryProperties.getTransitionType(trans);
	trans_type_.setSelectedItem(ttype == null ?
				    GeometryProperties.LINE : ttype);
	label_.setText(trans.get_label());

 	label_posit_.setText(Integer.toString
			     (GeometryProperties.getLabelPosition(trans)));

	// Handle advanced tab.
	if (trans.dst_get() == trans.src_get())
	{
	    loop_size_.setEnabled(true);
	    loop_angle_.setEnabled(true);
	    get_loop_angle(trans);
	    get_loop_size(trans);
	}
	else
	{
	    loop_size_.setEnabled(false);
	    loop_angle_.setEnabled(false);
	}
    }


    // Used functions for advanced tab (loops).
    public void			set_loop_angle(Transition t)
    {
	String selected = (String) loop_angle_.getSelectedItem();

	if (selected == "W")
	    GeometryProperties.setLoopDirection(t, 0);
	else if (selected == "NW")
	    GeometryProperties.setLoopDirection(t, 45);
	else if (selected == "N")
	    GeometryProperties.setLoopDirection(t, 90);
	else if (selected == "NE")
	    GeometryProperties.setLoopDirection(t, 135);
	else if (selected == "E")
	    GeometryProperties.setLoopDirection(t, 180);
	else if (selected == "SE")
	    GeometryProperties.setLoopDirection(t, 225);
	else if (selected == "S")
	    GeometryProperties.setLoopDirection(t, 270);
	else if (selected == "SW")
	    GeometryProperties.setLoopDirection(t, 315);
    }


    public void			set_loop_size(Transition t)
    {
	String size = (String) loop_size_.getSelectedItem();

	if (size.equals("Small"))
	    t.loop_size_set(VJToolKitConstants.LOOP_SIZE_SMALL);
	else
	    t.loop_size_set(VJToolKitConstants.LOOP_SIZE_NORMAL);
    }


    public void			get_loop_angle(Transition t)
    {
	int angle = GeometryProperties.getLoopDirection(t);

	if (angle == 0)
	    loop_angle_.setSelectedItem("W");
	else if (angle == 135)
	    loop_angle_.setSelectedItem("NE");
	else if (angle == 90)
	    loop_angle_.setSelectedItem("N");
	else if (angle == 45)
	    loop_angle_.setSelectedItem("NW");
	else if (angle == 90)
	    loop_angle_.setSelectedItem("E");
	else if (angle == 315)
	    loop_angle_.setSelectedItem("SW");
	else if (angle == 270)
	    loop_angle_.setSelectedItem("S");
	else if (angle == 225)
	    loop_angle_.setSelectedItem("SE");
    }


    public void			get_loop_size(Transition t)
    {
	if (t.loop_size_get() == VJToolKitConstants.LOOP_SIZE_NORMAL)
	    loop_size_.setSelectedItem("Normal");
	else
	    loop_size_.setSelectedItem("Small");
    }



    // Swing components for the frame.
    protected JComboBox		src_;
    protected JComboBox		dst_;
    protected JTextField	label_;

    // For advanced attributes.
    protected JComboBox		loop_angle_;
    protected JComboBox		loop_size_;
    protected JComboBox		trans_type_;
    protected JTextField	label_posit_;


    // Attributes.
    protected Transition	trans_;

}
