//
// GeometryProperties.java for vgi in /home/loulou/lrde_proj/vgi/src/vgitoolkit
//
// Made by Louis-Noel Pouchet
// Login   <loulou@lrde.epita.fr>
//
// Started on  Sun Mar  6 22:39:10 2005 Louis-Noel Pouchet
// Last update Mon Oct 30 22:09:22 2006 Guillaume
//

package vaucanson.vgitoolkit;

import java.util.Map;
import java.lang.Integer;
import java.awt.geom.Point2D;



// This class references all geometry properties that can be in an
// Vaucanson-XML document, regarding the DTD. Here we save:
// - an attribute map for the whole document, locally overriden by:
// - an attribute map for states or transitions, globally, that is
//   locally overriden by:
// - an attribute map for each state of transition.
public class GeometryProperties
{
    // XSD constants.
    public final static String X = "x";
    public final static String Y = "y";

    public final static void setX(State s, int x)
    {
        Map geometry = s.geometry_get();
        Integer xs = new Integer(x);

	s.modified_map_ = true;
        geometry.put(GeometryProperties.X, xs.toString());
    }

    public final static int getX(State s)
    {
        Map geometry = s.geometry_get();
        String str = (String)geometry.get(GeometryProperties.X);

        if (str != null)
        {
            Integer x = new Integer(str);
            return x.intValue();
        }
        else
            return 0;
    }

    public final static int getY(State s)
    {
        Map geometry = s.geometry_get();
        String str = (String)geometry.get(GeometryProperties.Y);

        if (str != null)
        {
            Integer x = new Integer(str);
            return x.intValue();
        }
        else
            return 0;
    }

    public final static void setY(State s, int x)
    {
        Map geometry = s.geometry_get();
        Integer xs = new Integer(x);

	s.modified_map_ = true;
        geometry.put(GeometryProperties.Y, xs.toString());
    }

    public final static void setXY(State s, Point2D p)
    {
        setX(s, (int)p.getX());
        setY(s, (int)p.getY());
    }



    // Transition type: describe how the transition should be displayed
    public final static String TRANSITIONTYPE = "transitionType";

    public final static void setTransitionType(Transition e, String type)
    {
        Map geometry = e.geometry_get();

        if (type != null || type.equals(LINE) || type.equals(ARCL) ||
                type.equals(ARCR) || type.equals(CURVE))
            geometry.put(TRANSITIONTYPE, type);
        else
            geometry.put(TRANSITIONTYPE, TRANSITIONTYPEDEFAULT);
    }

    public final static String getTransitionType(Transition e)
    {
        Map geometry = e.geometry_get();
        String type = (String) geometry.get(TRANSITIONTYPE);
        return type;
    }

    // Values for transitionType
    public final static String LINE = "line";
    public final static String ARCL = "arcL";
    public final static String ARCR = "arcR";
    public final static String CURVE = "curve";
    public final static String TRANSITIONTYPEDEFAULT = LINE;



    // Label Position: value from 0 to 100
    public final static String LABELPOSITION = "labelPosition";
    public final static int LABELPOSITIONDEFAULT = 50;

    public final static int getLabelPosition(TransitionTemplate e)
    {
        Map geometry = e.geometry_get();
        String str = (String) geometry.get(GeometryProperties.LABELPOSITION);

        if (str != null)
        {
            Integer lpos = new Integer(str);
            return lpos.intValue();
        }
        else
            return LABELPOSITIONDEFAULT;
    }

    public final static void setLabelPosition(TransitionTemplate e, int pos)
    {
        if (pos < 0 || pos > 100)
            pos = LABELPOSITIONDEFAULT;

        Map geometry = e.geometry_get();
        Integer ipos = new Integer(pos);

        geometry.put(GeometryProperties.LABELPOSITION, ipos.toString());
	e.label_is_set = false;
    }



    // Loop direction: angle describing where the loop is on the state
    public final static String LOOPDIRECTION = "loopDirection";
    public final static int LOOPDIRECTIONDEFAULT = 0;

    public final static int getLoopDirection(Transition e)
    {
        Map geometry = e.geometry_get();
        String str = (String) geometry.get(GeometryProperties.LOOPDIRECTION);

        if (str != null)
        {
            Integer idir = new Integer(str);
            return idir.intValue();
        }
        else
            return -1;
    }

    public final static void setLoopDirection(Transition e, int dir)
    {
        if (dir < 0 || dir > 360)
            dir = LOOPDIRECTIONDEFAULT;

        Map geometry = e.geometry_get();
        Integer idir = new Integer(dir);

        geometry.put(GeometryProperties.LOOPDIRECTION, idir.toString());
    }



    // Angle for final and initial transitions
    public final static String TRANSITIONANGLE = "transitionAngle";
    public final static int TRANSITIONANGLEDEFAULT = 0;

    public final static int getTransitionAngle(TerminalTransition e)
    {
        Map geometry = e.geometry_get();
        String str = (String) geometry.get(GeometryProperties.TRANSITIONANGLE);

        if (str != null)
        {
            Integer iangle = new Integer(str);
            return iangle.intValue();
        }
        else
            return -1;
    }

    public final static void setTransitionAngle(TerminalTransition e, int a)
    {
        if (a < 0 || a >= 360)
            a = TRANSITIONANGLEDEFAULT;

        Map geometry = e.geometry_get();
        Integer ia = new Integer(a);

        geometry.put(GeometryProperties.TRANSITIONANGLE, ia.toString());
    }


    // Deprecated !
    public final static String DIRECTION = "direction";
    public final static String ORIENTATION = "orientation";

}
