//
// LayoutTemplate.java for Vcsn in /home/loulou/enst/VcsnInterface
//
// Made by Loulou Pouchet
// Login   <loulou@lrde.epita.fr>
//
// Started on  Thu Dec  9 11:12:01 2004 Loulou Pouchet
// Last update Tue Dec  5 21:39:39 2006 Guillaume
//

package vaucanson.vgitoolkit;


import java.awt.geom.Rectangle2D;
import java.awt.geom.Point2D;
import java.util.ArrayList;
import java.util.List;
import java.util.ListIterator;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Map;
import java.util.Set;

import org.jgraph.JGraph;
import org.jgraph.graph.DefaultGraphModel;
import org.jgraph.graph.CellView;
import org.jgraph.graph.EdgeView;
import org.jgraph.graph.GraphConstants;
import org.jgraph.graph.GraphLayoutCache;
import org.jgraph.graph.VertexView;

import org.jgraph.util.*;
import org.jgraph.layout.*;


// Base class for all layouts. This code is ugly.
public abstract class LayoutTemplate
{
    public VJGraph		initialize_layout(VJGraph source_graph)
    {
      Set layout_attributes = new HashSet();

      layout_attributes.add(GraphConstants.BOUNDS);
      layout_attributes.add(GraphConstants.POINTS);
      layout_attributes.add(GraphConstants.LABELPOSITION);
      layout_attributes.add(GraphConstants.ROUTING);

      VJGraph local_graph = new VJGraph(source_graph.getModel());
      local_graph.setBounds(source_graph.getBounds());
      cache_ = local_graph.getGraphLayoutCache();
      cache_.setLocalAttributes(layout_attributes);
      source_graph_ = source_graph;

      return local_graph;
    }


    public void			finalize_layout()
    {
      Map nested = new Hashtable();
      CellView[] cellViews = cache_.getAllDescendants(cache_.getRoots());

      for (int i = 0; i < cellViews.length; i++)
      {
	Map attrs = cellViews[i].getAttributes();
	Rectangle2D bounds = GraphConstants.getBounds(attrs);

	if (cellViews[i] instanceof StateView && bounds == null)
	{
	  bounds =  cellViews[i].getBounds();
	  GraphConstants.setBounds(attrs, bounds);
	  State s = (State) cellViews[i].getCell();
	  GeometryProperties.setX(s, (int)bounds.getX());
	  GeometryProperties.setY(s, (int)bounds.getY());
	}
	if (!attrs.isEmpty())
	  nested.put(cellViews[i].getCell(), attrs);
      }
      if (!nested.isEmpty())
	source_graph_.getModel().edit(nested, null, null, null);
    }

    public void			finalize_layout(List vertices)
    {
	for (int i = 0; i < vertices.size(); ++i)
	{
	    Map attrs = new Hashtable();
	    State s = (State) vertices.get(i);
	    s.update();
	}
    }

    // Set force to true if you want all vertices to be retrieved, to false if
    // you want to leave unchanged vertices with initial position.
    public static List		get_vertices(VJGraph graph,
					     Object[] cells,
					     boolean force)
    {
      // Fetch all State
      State[] states = VJGraph.get_states(graph);
      List vertices = new ArrayList();
      if (cells != null)
      {
	for (int i = 0; i < cells.length; i++)
	{
	  if (cells[i] instanceof State)
	    vertices.add(cells[i]);
	}
	return vertices;
      }

      for (int i = 0; i < states.length; i++)
      {
	if (force)
	  vertices.add(states[i]);
	else
	{
	  Map m = states[i].geometry_get();
	  if (m == null ||
	      m.get(GeometryProperties.X) == null ||
	      m.get(GeometryProperties.Y) == null)
	    vertices.add(states[i]);
	}
      }
      return  vertices;
    }

    public static List		get_all_vertices(VJGraph graph)
    {
      return get_vertices(graph, null, true);
    }


    // Set force to true if you want all transitions to be retrieved, to false if
    // you want to leave unchanged transition with initial settings.
    public static List		get_transitions(VJGraph graph,
						Object[] cells,
						boolean force)
    {
      Transition[] trans = VJGraph.get_transitions(graph);
      List transitions = new ArrayList();

      if (cells != null)
      {
	for (int i = 0; i < cells.length; i++)
	  if (cells[i] instanceof Transition)
	    transitions.add(cells[i]);
	return transitions;
      }
      for (int i = 0; i < trans.length; i++)
      {
	if (force)
	  transitions.add(trans[i]);
	else
	{
	  Map m = trans[i].geometry_get();
	  if (m == null ||
	      GeometryProperties.getTransitionType(trans[i]) == null)
	    transitions.add(trans[i]);
	}
      }
      return transitions;
    }

    public static List		get_all_transitions(VJGraph graph)
    {
      return get_transitions(graph, null, true);
    }

    // All layouts must define this method.
    public abstract void	apply(VJGraph graph, Object[] cells);

    public static Object[] get_parallel_transitions(VJGraph g,
						    Transition edge,
						    boolean directed)
    {
      return DefaultGraphModel.getEdgesBetween(g.getModel(),
					       edge.getSource(),
					       edge.getTarget(),
					       directed);
    }


    public static void apply_to_transitions(VJGraph g, List trans)
    {
      for (ListIterator itl = trans.listIterator(); itl.hasNext();)
      {
	Transition e = (Transition) itl.next();
	Object[] parallels = get_parallel_transitions(g, e, false);

	// If there is only one edges (i.e normally just our current e)
	if (parallels.length < 2)
	{
	  GeometryProperties.setTransitionType(e, GeometryProperties.LINE);
	  return;
	}

	// else find position of transition e, if it's even, e is set to ARCL
	// else to ARCR
	int epos = 0;
	for (; epos < parallels.length; epos++)
	  if (parallels[epos] instanceof Transition && e == parallels[epos])
	    break;
	if ((epos % 2) != 0)
	  GeometryProperties.setTransitionType(e, GeometryProperties.ARCR);
	else
	  GeometryProperties.setTransitionType(e, GeometryProperties.ARCL);
      }
    }


    // Global layout, called at each initialisation.
    public static void		apply_layout(VJGraph graph, boolean unused)
    {
      Object[] cells = (graph.isSelectionEmpty()) ?
	null :
	graph.getSelectionCells();
      GlobalLayout layout = new GlobalLayout();
      layout.apply(graph, cells);
      apply_to_transitions(graph, get_transitions(graph, null, false));

      // Center the graph
      CenterGraphLayout clayout = new CenterGraphLayout();
      clayout.apply(graph, null);
      Tools.refresh_view(graph);
    }

    // Global layout, called at each initialisation.
    /// FIXME: Should take cells instead, in order to leave cells which have
    /// geometric properties set to their original places.
    public static void		apply_global_layout(VJGraph graph)
    {
      // Put states on a line.
      QuadGraphLayout layout = new QuadGraphLayout();
      Object[] sel = VJGraph.get_selection(graph);
      layout.apply(graph, DefaultGraphModel.getAll(graph.getModel()), 1);

      // If there is more than 5 states, launch the circle layout algorithm.
      if (VJGraph.get_states(graph).length > 5)
      {
	CircleGraphLayout cllayout = new CircleGraphLayout();
	cllayout.apply(graph,
		       DefaultGraphModel.getAll(graph.getModel()));
      }

      // "Center" the graph.
      CenterGraphLayout clayout = new CenterGraphLayout();
      clayout.apply(graph, DefaultGraphModel.getAll(graph.getModel()));

      // Set geometry of transitions
      apply_to_transitions(graph, get_all_transitions(graph));

      Tools.refresh_view(graph);
    }



    // Attributes.
    private GraphLayoutCache	cache_;
    protected VJGraph		source_graph_ = null;
}
