//
// LoopRouter.java for vgi in /home/loulou/lrde_proj/vgi/src/vgi
//
// Made by Louis-Noel Pouchet
// Login   <loulou@lrde.epita.fr>
//
// Started on  Mon Jun 20 00:34:14 2005 Louis-Noel Pouchet
// Last update Wed Jun 22 02:02:34 2005 Louis-Noel Pouchet
//

package vaucanson.vgitoolkit;

import java.util.Map;
import java.util.Hashtable;
import java.util.List;
import java.awt.geom.Point2D;
import java.awt.Point;
import java.awt.geom.Rectangle2D;

import org.jgraph.graph.*;
import org.jgraph.graph.GraphConstants;


public class LoopRouter implements Edge.Routing {

    public int selfReferenceSize = 20;


    /**
     * @param edge and edge View
     *
     * @return the preferred line style (here BEZIER)
     */
    public int getPreferredLineStyle(EdgeView edge)
    {
	return GraphConstants.STYLE_SPLINE;
    }


    protected double		get_loop_angle(EdgeView edge)
    {
	int angle;

	Transition trans = (Transition) edge.getCell();
	if (GeometryProperties.getLoopDirection(trans) == -1 ||
	    VJToolKitConstants.REDRAW)
	{
	    angle = GeometricUtilities.get_free_angle(trans.graph_get(),
						      trans.src_get(),
						      trans);
	    GeometryProperties.setLoopDirection(trans, angle);
	}
	else
	    angle = GeometryProperties.getLoopDirection(trans);
	return ((double)angle) * Math.PI / 180;
    }


    public List route(EdgeView edge)
    {
	List points = edge.getPoints();
	double alpha = get_loop_angle(edge);
	int n = points.size();
	Point2D from = edge.getPoint(0);
	if (edge.getSource() instanceof PortView)
	    from = ((PortView) edge.getSource()).getLocation(null);
	else if (edge.getSource() != null)
	{
	    Rectangle2D b = edge.getSource().getBounds();
	    from =
		edge.getAttributes().createPoint(b.getCenterX(),
						 b.getCenterY());
	}
	Point2D to = edge.getPoint(n - 1);
	if (edge.getTarget() instanceof PortView)
	    to = ((PortView) edge.getTarget()).getLocation(null);
	else if (edge.getTarget() != null)
	{
	    Rectangle2D b = edge.getTarget().getBounds();
	    to = edge.getAttributes().createPoint(b.getCenterX(),
						  b.getCenterY());
	}
	if (from != null && to != null)
	{
	    Point2D[] routed;
	    // Handle self references
	    if (edge.getSource() == edge.getTarget()
		&& edge.getSource() != null)
	    {
		Rectangle2D bounds =
		    edge.getSource().getParentView().getBounds();
		double height = selfReferenceSize;
		double width = bounds.getWidth();
		routed = new Point2D[2];
		routed[0] = edge.getAttributes()
		    .createPoint(bounds.getX() - width / 5,
				 bounds.getY() + bounds.getHeight() +
				 height);

		routed[1] =
		    edge.getAttributes().createPoint(bounds.getX() +
						     width,
						     bounds.getY() +
						     bounds.getHeight()
						     + height);
		GeometricUtilities.rotate(from, routed[0],
					  alpha -  Math.PI / 2);
		GeometricUtilities.rotate(from, routed[1],
					  alpha -  Math.PI / 2);

		// Set/Add Points
		for (int i = 0; i < routed.length; i++)
		    if (points.size() > i + 2)
			points.set(i + 1, routed[i]);
		    else
			points.add(i + 1, routed[i]);
	    }
	}
	return points;
    }

}
