//
// QuadGraphLayout.java for Vcsn in /home/loulou/enst/VcsnInterface
//
// Made by Loulou Pouchet
// Login   <loulou@lrde.epita.fr>
//
// Started on  Thu Dec  9 10:06:56 2004 Loulou Pouchet
// Last update Fri Dec  8 02:08:20 2006 Guillaume
//

package vaucanson.vgitoolkit;


import java.awt.geom.Rectangle2D;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Map;
import java.util.Set;
import java.util.ArrayList;
import java.util.List;

import org.jgraph.JGraph;
import org.jgraph.graph.CellView;
import org.jgraph.graph.VertexView;


// This basic layout put states on a grid, where STEP_X and STEP_Y are
// step for x and y axis.
// This layout also handle fixed states.
public class QuadGraphLayout
  extends LayoutTemplate
{
    public static final int STEP_X = 4;
    public static final int STEP_Y = 4;


    public void			apply(VJGraph source_graph,
				      Object[] cells)
    {
      int x = (int) Math.floor(Math.sqrt(cells.length));
      apply(source_graph, cells, x, true);
    }


    public void			apply(VJGraph source_graph,
				      Object[] cells,
				      int x)
    {
      apply(source_graph, cells, x, true);
    }


    public void			apply(VJGraph source_graph,
				      Object[] cells,
				      int x,
				      boolean force)
    {
      List vertices;

      vertices = get_vertices(initialize_layout(source_graph), cells, force);
      if (x == 0)
	x = (int) Math.ceil(Math.sqrt(vertices.size()));
      // Run the layout algorithm.
      run_layout(vertices, x);
      // Finalize the algorithm by updating current graph with new values.
      finalize_layout(vertices);
    }


    public void			apply_at_start(VJGraph source_graph,
					       Object[] cells,
					       int x)
    {
      List vertices;

      vertices = get_vertices(initialize_layout(source_graph), cells, true);

      // Run the layout algorithm.
      run_layout(vertices, (int) Math.ceil(Math.sqrt(vertices.size())));
      // Finalize the algorithm by updating current graph with new values.
      finalize_layout(vertices);
    }


    public static void		run_layout(List vertices, int nl)
    {
      // nl represents the number of lines.
      if (nl < 1)
	return;

      int sx = 1, sy = 1, i, cpt = 0;
      double offsetX = 0;
      double offsetY = 0;

      for (i = 0; i < vertices.size(); i++)
      {
	State s = (State) vertices.get(i);

	offsetX += GeometryProperties.getX(s);
	offsetY += GeometryProperties.getY(s);
	cpt++;
      }

      offsetY /= cpt;
      if (nl != 1)
	offsetX /= cpt;
      else
      {
	i = 0;
	while (i < vertices.size() &&
	       GeometryProperties.getX((State) vertices.get(i)) == 0)
	  ++i;
	if (i < vertices.size())
	  offsetX = GeometryProperties.getX((State) vertices.get(i));
      }

      for (i = 0; i < vertices.size(); i++)
      {
	State s = (State) vertices.get(i);
	sy = i % nl;
	sx = i / nl;
	GeometryProperties.setX(s, (int)(STEP_X * sx + offsetX));
	GeometryProperties.setY(s, (int)(STEP_Y * sy + offsetY));
      }
    }

}
