
//State.java for vgi in /home/loulou/lrde_proj/vgi/src

//Made by Louis-Noel Pouchet
//Login   <loulou@lrde.epita.fr>

//Started on  Wed Mar  2 14:36:23 2005 Louis-Noel Pouchet
//Last update Fri Jun 24 09:23:08 2005 Loulou Pouchet



package vaucanson.vgitoolkit;


import org.jgraph.JGraph;
import org.jgraph.graph.*;

import java.util.Map;
import java.util.Hashtable;
import java.util.List;

import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;


//Vaucanson implementation of DefaultGraphCell. Used for each states.
public class State extends DefaultGraphCell
{
	State()
	{
		super();
	}

	State(Object obj)
	{
		super(obj);
	}


	public State(String name, Map geometry, GraphModel model, VJGraph graph)
	{
		super();
		// FIXME: Use number factory instead of 0.
		create_state(name, 0, name, geometry, model, graph);
	}


	public State(String name,
			int number,
			String label,
			Map geometry,
			GraphModel model,
			VJGraph graph)
	{
		super();
		create_state(name, number, label, geometry, model, graph);
	}


	protected void			create_state(String name,
			int number,
			String label,
			Map geometry,
			GraphModel model,
			VJGraph graph)
	{
		graph_ = graph;
		this.name_ = name;
		this.number_ = number;
		this.label_ = label;
		this.geometry_map_ = geometry;
		graph_model_ = model;
		port_ = new DefaultPort();
		port_.setUserObject(this);
		this.add(port_);
		VJTools.init_jgraph_attributes(this);
		set_geometry_coord();
	}


	public boolean			is_final()
	{
		return final_transition_ != null;
	}


	public boolean			is_initial()
	{
		return initial_transition_ != null;
	}


	public void				set_geometry_coord()
	{
		String x = (String) geometry_map_.get("x");
		String y = (String) geometry_map_.get("y");
		if (x != null && y != null)
		{
			Map attrib = new Hashtable();
			GraphConstants.setAbsolute(attrib, true);
			Rectangle2D bounds = GraphConstants.getBounds(attrib);
			Double valx = new Double(x);
			Double valy = new Double(y);
			if (bounds == null)
				bounds = new Rectangle2D.Double(valx.doubleValue(),
						valy.doubleValue(), 0, 0);
			else
				bounds.setRect(valx.doubleValue(), valy.doubleValue(),
						bounds.getWidth(), bounds.getHeight());
			GraphConstants.setBounds(attrib, bounds);
			Map map = new Hashtable();
			map.put(this, attrib);
			graph_model_.edit(map, null, null, null);
		}
	}


	public void				geometry_set(Map map)
	{
		geometry_map_ = map;
	}

	public Map				geometry_get()
	{
		return geometry_map_;
	}

	public int				number_get()
	{
		return number_;
	}

	public void				number_set(int n)
	{
		number_ = n;
	}


	public String			name_get()
	{
		return name_;
	}


	public void				label_set(String label)
	{
		label_ = label;
		update_label(label);
	}

	public String			label_get()
	{
		return label_;
	}

	public String			get_label()
	{
		return super.toString();
	}


	public void				graph_model_set(GraphModel graph_model)
	{
		graph_model_ = graph_model;
	}

	public GraphModel			graph_model_get()
	{
		return graph_model_;
	}


	public TerminalTransition		initial_transition_get()
	{
		return initial_transition_;
	}


	public TerminalTransition		final_transition_get()
	{
		return final_transition_;
	}


	public DefaultPort			port_get()
	{
		return port_;
	}


	public Map				get_attributes_map()
	{
		return graph_model_.getAttributes(this);
	}


	public void				set_initial(String label,
			Map geom)
	{
		set_initial(label, null, geom);
	}

	public void				set_initial(String label,
			String weight,
			Map geom)
	{
		if (label == null || label == "")
			// FIXME: Use semiring identity here !
			label = "";

		TerminalTransition trans =
			new TerminalTransition(this, label,
					VJToolKitConstants.INITIAL_TRANSITION,
					geom);

		trans.label_set(label);
		trans.weight_set(weight);

		initial_transition_ = trans;
	}

	// Shortcut.
	public void				update_initial_label(String label)
	{
		if (initial_transition_ != null)
			initial_transition_.label_set(label);
		else
			Debug.echo("State.java:Updating non initial state.");
	}

	// Shortcut.
	public void				update_initial_weight(String weight)
	{
		if (initial_transition_ != null)
			initial_transition_.weight_set(weight);
		else
			Debug.echo("State.java:Updating non initial state.");
	}


	public void				unset_initial()
	{
		if (initial_transition_ != null)
		{
			graph_model_.remove(new Object[]{initial_transition_});
			initial_transition_ = null;
		}
	}

	public void				set_final(String label,
			Map geom)
	{
		set_final(label, null, geom);
	}

	public void				set_final(String label,
			String weight,
			Map geom)
	{
		if (label == null || label == "")
			// FIXME: Use semiring identity here !
			label = "";

		TerminalTransition trans =
			new TerminalTransition(this, label,
					VJToolKitConstants.FINAL_TRANSITION,
					geom);

		trans.label_set(label);
		trans.weight_set(weight);

		final_transition_ = trans;
	}

	// Shortcut.
	public void				update_final_label(String label)
	{
		if (final_transition_ != null)
			final_transition_.label_set(label);
		else
			Debug.echo("State.java:Updating non final state.");
	}

	// Shortcut.
	public void				update_final_weight(String weight)
	{
		if (final_transition_ != null)
			final_transition_.weight_set(weight);
		else
			Debug.echo("State.java:Updating non final state.");
	}


	public void				unset_final()
	{
		if (final_transition_ != null)
		{
			graph_model_.remove(new Object[]{final_transition_});
			final_transition_ = null;
		}
	}


	public void				update_label(String label)
	{
		Map attrib = graph_model_.getAttributes(this);
		GraphConstants.setValue(attrib, label);
		this.label_ = label;
		Map map = new Hashtable();
		map.put(this, attrib);
		graph_model_.edit(map, null, null, null);
	}

	public void				update()
	{
		Map attr = graph_model_.getAttributes(this);

		// Update position
		Rectangle2D bounds = GraphConstants.getBounds(attr);
		if (bounds == null)
			bounds = new Rectangle2D.Double(0, 0, 10, 10);

		Point2D pos = new Point2D.Double(GeometryProperties.getX(this),
				GeometryProperties.getY(this));
		Point2D apos = GeometricUtilities.toScreen(graph_, pos);

		Rectangle2D new_bounds = new Rectangle2D.Double(apos.getX(), apos.getY(),
				bounds.getWidth(),
				bounds.getHeight());
		GraphConstants.setBounds(attr, new_bounds);
		Map nested = new Hashtable();
		nested.put(this, attr);
		graph_model_.edit(nested, null, null, null);
		modified_map_ = false;
	}

	public VJGraph			graph_get()
	{
		return graph_;
	}

	// Attributes.
	protected VJGraph			graph_;
	protected String			name_;
	protected String			label_;
	protected int			number_;
	protected DefaultPort		port_;

	protected TerminalTransition	initial_transition_ = null;
	protected TerminalTransition	final_transition_ = null;

	protected Map			geometry_map_;

	protected GraphModel		graph_model_;
	public boolean			modified_map_ = false;
}
