
//
// TransitionRouter.java for vgi in /home/loulou/lrde_proj/vgi/src
//
// Made by Louis-Noel Pouchet
// Login   <loulou@lrde.epita.fr>
//
// Started on  Wed Mar  2 17:03:21 2005 Louis-Noel Pouchet
// Last update Wed Nov 22 21:54:10 2006 Guillaume
//


package vaucanson.vgitoolkit;


import org.jgraph.util.*;


import java.awt.geom.Point2D;

import org.jgraph.graph.Edge;
import org.jgraph.graph.*;
import org.jgraph.graph.GraphConstants;
import org.jgraph.graph.EdgeView;
import org.jgraph.graph.PortView;

/// ABM
import java.awt.Point;
import java.lang.Double;
import java.util.Map;
import java.awt.geom.Rectangle2D;
import java.util.List;

import java.util.Hashtable;
import java.util.Iterator;
import java.util.Set;
import java.util.Vector;

import java.awt.Color;
import org.jgraph.util.Bezier;
import java.util.ArrayList;
import org.jgraph.JGraph;




public class TransitionRouter implements Edge.Routing
{
    protected static GraphModel emptyModel = new DefaultGraphModel();

    public static TransitionRouter sharedInstance = new TransitionRouter();

    /**
     * The distance between the control point and the middle line. A larger
     * number will lead to a more "bubbly" appearance of the bezier edges.
     */
    public double edgeSeparation = 20;

    // 	private TransitionRouter() {
    // 		// empty
    // 	}

    /**
     * Returns the array of parallel edges.
     *
     * @param edge
     * @return
     */
    public Object[] getParallelEdges(EdgeView edge)
    {
	// FIXME: The model is stored in the cells only in the default
	// implementations. Otherwise we must use the real model here.
	return
	    DefaultGraphModel.getEdgesBetween(emptyModel, edge.getSource()
					      .getParentView().getCell(),
					      edge.getTarget().getParentView()
					      .getCell(), false);
    }

    public List route(EdgeView edge)
    {
	Transition t = (Transition) edge.getCell();
	String trans_t = GeometryProperties.getTransitionType(t);
	List points = edge.getPoints();

	if (edge.getSource() == null || edge.getTarget() == null
	    || edge.getSource().getParentView() == null
	    || edge.getTarget().getParentView() == null)
	    return points;


	if (trans_t == null)
	    trans_t = GeometryProperties.LINE;
	// If there is only 1 edge between the two vertices, we don't need this
	// special routing
	if (trans_t.equals(GeometryProperties.LINE))
	{
	    if (points.size() > 2)
		points.remove(1);

	    return points;
	}

	Object[] edges = getParallelEdges(edge);
	// Find the position of the current edge that we are currently routing
	if (edges == null)
	    return points;

	ArrayList like_me = new ArrayList();
	for (int i = 0; i < edges.length; i++)
	{
	    Transition e = (Transition) edges[i];
	    String e_type = GeometryProperties.getTransitionType(e);

	    if (e_type == null || e_type.equals(GeometryProperties.LINE))
		continue;
	    // edges have opposed direction and differenct arc type
	    if (e.getTarget() == t.getSource() && !e_type.equals(trans_t))
	    {
		like_me.add(e);
		continue;
	    }
	    if (e.getTarget() == t.getTarget() && trans_t.equals(e_type))
		like_me.add(e);
	}

	int position = 0;
	for (int i = 0; i < like_me.size(); i++)
	{
	    Transition e = (Transition) like_me.get(i);
	    if (e == edge.getCell())
		position = i;
	}


	// Find the end point positions
	int n = points.size();
	Point2D from = ((PortView) edge.getSource()).getLocation(null);
	Point2D to = ((PortView) edge.getTarget()).getLocation(null);

	if (from != null && to != null)
	{
	    Point2D controlPoint = null;
	    double d = edgeSeparation * (position + 1);
	    if (trans_t.equals(GeometryProperties.ARCL))
		controlPoint = GeometricUtilities.get_point_on_normal(from, to, d);
	    else
		controlPoint = GeometricUtilities.get_point_on_normal(to, from, d);

	    // add the control point to the points list
	    if (points.size() == 2)
		points.add(1, controlPoint);
	    else
		points.set(1, controlPoint);

	}
	return points;
    }

    /**
     * @return Returns the edgeSeparation.
     */
    public double getEdgeSeparation()
    {
	return edgeSeparation;
    }

    /**
     * @param edgeSeparation
     *            The edgeSeparation to set.
     */
    public void setEdgeSeparation(double edgeSeparation)
    {
	this.edgeSeparation = edgeSeparation;
    }

    /**
     * @return Returns the sharedInstance.
     */
    public static TransitionRouter getSharedInstance()
    {
	return sharedInstance;
    }

    /**
     * @param edge and edge View
     *
     * @return the preferred line style (here BEZIER)
     */
    public int getPreferredLineStyle(EdgeView edge)
    {
	return GraphConstants.STYLE_SPLINE;
    }

    public static double	      set_label_offset(EdgeView edge, double offset)
    {
	Transition t = (Transition) edge.getCell();
	Point2D from = ((PortView) edge.getSource()).getLocation(null);
	Point2D to = ((PortView) edge.getTarget()).getLocation(null);
	String trans_type = GeometryProperties.getTransitionType(t);
	double dx = from.getX() - to.getX();

	if (trans_type == null || trans_type.equals(GeometryProperties.LINE))
	{
	    if (dx >= 0)
		return -offset;
	    else
		return offset;
	}
	if (trans_type.equals(GeometryProperties.ARCR))
	{
	    if (dx >= 0)
		return offset;
	    else
		return -offset;
	}

	if (trans_type.equals(GeometryProperties.ARCL))
	{
	    if (dx >= 0)
		return offset;
	    else
		return -offset;
	}
	return 0;
    }



}
