//
// TransitionView.java for vgi in src/vgitoolkit
//
// Made by Guillaume Leroi
// Login   <guillaume.leroi@lrde.epita.fr>
//


package vaucanson.vgitoolkit;


import java.awt.BasicStroke;
import java.awt.Dimension;
import java.awt.GradientPaint;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.awt.Rectangle;

import java.util.ArrayList;
import java.util.Map;

import org.jgraph.JGraph;
import org.jgraph.graph.AttributeMap;
import org.jgraph.graph.CellViewRenderer;
import org.jgraph.graph.Edge;
import org.jgraph.graph.EdgeRenderer;
import org.jgraph.graph.EdgeView;
import org.jgraph.graph.GraphConstants;
import org.jgraph.graph.GraphModel;
import org.jgraph.util.Spline2D;

public class TransitionView extends EdgeView
{

    public static transient TransitionRenderer renderer = new TransitionRenderer();

    public CellViewRenderer getRenderer() {
      return renderer;
    }


    public TransitionView()
    {
      super();
    }

    public TransitionView(Object cell) {
      super(cell);
    }





    public static class TransitionRenderer extends EdgeRenderer
    {

	protected void paintLabel(Graphics g, String label, Point2D p,
				  boolean mainLabel)
	{
	  if (g != null && p != null && mainLabel)
	  {
	    Graphics2D g2 = (Graphics2D) g;
	    int sw = metrics.stringWidth(label);
	    int sh = metrics.getHeight();

	    // Get the label position
	    if (view.getPointCount() <= 0)
	      return;
	    Debug.echo(view.getPointCount());
	    Point2D[] pts = new Point2D[view.getPointCount()];
	    for (int i = 0; i < view.getPointCount(); i++)
	      pts[i] = view.getPoint(i);
	    Spline2D b = new Spline2D(pts);
	    Transition t = (Transition) view.getCell();
	    double lpos = ((double)GeometryProperties.getLabelPosition(t))/100;

	    // Compute absolute position with offset
	    double[] spos = b.getPoint(lpos - 0.2 >= 0 ? lpos - 0.2 : 0);
	    double[] epos = b.getPoint(lpos + 0.2 <= 1 ? lpos + 0.2 : 1);
	    Point2D from = new Point2D.Double(spos[0], spos[1]);
	    Point2D to = new Point2D.Double(epos[0], epos[1]);
	    String trans_t = GeometryProperties.getTransitionType(t);
	    Point2D label_pos = null;
	    if (trans_t != null && trans_t.equals(GeometryProperties.ARCR))
	      label_pos = GeometricUtilities.
		get_point_on_normal(to, from,
				    DrawingProperties.getLabelOffset(t));
	    else
	      label_pos = GeometricUtilities.
		get_point_on_normal(from, to,
				    DrawingProperties.getLabelOffset(t));

	    //Extend the clipping area
	    Rectangle2D clip = g.getClipBounds();
	    double width = Math.max(clip.getX() + clip.getWidth(),
				    label_pos.getX() + sw);
	    double height = Math.max(clip.getY() + clip.getHeight(),
				     label_pos.getY() + sh);
	    g.setClip((int)Math.round(clip.getX()),
		      (int)Math.round(clip.getY()),
		      (int)Math.round(width), (int)Math.round(height));

	    // Move to label position to draw it
	    g2.translate(label_pos.getX(), label_pos.getY());
	    if (isOpaque() && mainLabel)
	    {
	      g.setColor(getBackground());
	      g.fillRect(-sw / 2 - 1, -sh / 2 - 1, sw + 2, sh + 2);
	    }
// 	    if (borderColor != null && mainLabel)
// 	    {
// 	      g.setColor(borderColor);
// 	      g.drawRect(0, 0, sw + 2, sh + 2);
// 	    }
	    g.setColor(fontColor);
	    g.drawString(label, 0, sh);
	    g2.translate(-label_pos.getX(), -label_pos.getY());
	  }
	}

    }


}
