//
// VJGraph.java for vgi in /home/loulou/lrde_proj/vgi/src
//
// Made by Louis-Noel Pouchet
// Login   <loulou@lrde.epita.fr>
//
// Started on  Wed Mar  2 14:19:54 2005 Louis-Noel Pouchet
// Last update Tue Oct 16 15:32:09 2007 Guillaume
//


package vaucanson.vgitoolkit;


import org.jgraph.util.*;

import org.jgraph.JGraph;
import org.jgraph.graph.*;
import org.jgraph.*;

import java.awt.geom.Rectangle2D;
import java.awt.geom.Point2D;
import java.awt.geom.Dimension2D;

import java.util.Map;
import java.util.Vector;
import java.util.Hashtable;
import java.util.Set;
import java.util.Iterator;

import java.awt.Color;

import org.w3c.dom.*;


import org.jgraph.graph.GraphUndoManager;


// Vaucanson extension of JGraph class. Enables overriding of viewing methods.
public class VJGraph extends JGraph
{

    // Default constructor.
    public VJGraph()
    {
      super();
      automaton_structure_ = new Hashtable();
      init_automaton_structure();

      viewport_x = 0;
      viewport_y = 0;
    }

    //
    public VJGraph(GraphModel obj)
    {
      super(obj);
      automaton_structure_ = new Hashtable();
      init_automaton_structure();

      viewport_x = 0;
      viewport_y = 0;
    }


    // Special undo functions.
    public void			add_undo_manager()
    {
      undo_manager_ = new GraphUndoManager();
      getModel().addUndoableEditListener(undo_manager_);
    }

    public GraphUndoManager	undo_manager_get()
    {
      return undo_manager_;
    }

    protected GraphUndoManager	undo_manager_;



    // Create new transition.
    public static Transition		create_transition(VJGraph graph,
							  State s1,
							  State s2)
    {
      return create_transition(graph, s1, s2,
			       graph.automaton_geometry_get());
    }

    // Create new transition.
    public static Transition		create_transition(VJGraph graph,
							  State s1,
							  State s2,
							  Map geom)
    {
      Map geometry = Tools.merge_map(graph.automaton_geometry_get(), geom);
      Transition transition = new Transition(s1, s2,
					     geometry,
					     graph.getModel(),
					     graph);
      return transition;
    }



    // Create new state.
    public static State			create_state(VJGraph graph,
						     String name)
    {
      return create_state(graph, name, graph.automaton_geometry_get());
    }

    // Create new state.
    public static State			create_state(VJGraph graph,
						     String name,
						     Map geom)
    {
      Map geometry = Tools.merge_map(graph.automaton_geometry_get(), geom);
      State state = new State(name, geometry, graph.getModel(), graph);
      return state;
    }


    // Delete transition.
    public static void			delete_transition(VJGraph graph,
							  Transition t)
    {
      graph.getModel().remove(new Object[]{t});
    }


    // Delete state.
    public static void			delete_state(VJGraph graph,
						     State s)
    {
      graph.getModel().remove(new Object[]{s});
    }


    // XML <type> tag related stuff.
    //
    // Special attribute containing the XML type of the current automaton.
    // See VJToolKitConstants.java for reserved XML tag constants.
    protected Map			automaton_structure_;
    protected String			name_ = "";

    // Accessor.
    public Map				automaton_structure_get()
    {
      return automaton_structure_;
    }

    public String			name_get()
    {
      return name_;
    }

    public void				name_set(String name)
    {
      name_ = name;
    }

    public void				set_name(NodeList l)
    {
      for (int i = 0; i < l.getLength(); i++)
      {
	Node elt = l.item(i);
	if (elt.getNodeType() == Node.ELEMENT_NODE &&
	    (elt.getNodeName().equals("automaton") ||
	     elt.getNodeName().equals("transducer")))
	{
	  NamedNodeMap attr = elt.getAttributes();
	  if (attr.getNamedItem("name") != null)
	    name_ = attr.getNamedItem("name").getNodeValue();
	}
      }
    }


    // Set automaton algebraic structure to default values. (See DTD.)
    public void				init_automaton_structure()
    {
      automaton_structure_.put(VJToolKitConstants.XML_AUTOMATON_TYPE,
			       VJToolKitConstants.XML_AUTOMATON_TYPE_DEFAULT);
      automaton_structure_.put(VJToolKitConstants.XML_ALPHA_IN,
			       VJToolKitConstants.XML_ALPHA_IN_DEFAULT);
      automaton_structure_.put(VJToolKitConstants.XML_ALPHA_OUT,
			       VJToolKitConstants.XML_ALPHA_OUT_DEFAULT);
      automaton_structure_.put(VJToolKitConstants.XML_WEIGHT,
			       VJToolKitConstants.XML_WEIGHT_DEFAULT);
    }


    protected String			get_weight(NodeList l)
    {
      NamedNodeMap			attr;
      String res = "B";

      for (int i = 0; i < l.getLength(); i++)
      {
	Node elt = l.item(i);
	if (elt.getNodeType() == Node.ELEMENT_NODE &&
	    elt.getNodeName().equals("semiring"))
	{
	  attr = elt.getAttributes();
	  String set = "";
	  if (attr.getNamedItem("set") != null)
	    set = attr.getNamedItem("set").getNodeValue();
	  String operations = "";
	  if (attr.getNamedItem("operations") != null)
	    operations =
	      attr.getNamedItem("operations").getNodeValue();
	  if (operations.equals("tropicalMin"))
	    res =  "(Z,min,+)";
	  else if (operations.equals("tropicalMax"))
	    res = "(Z,max,+)";
	  else if (set == "" || set.equals("B"))
	    res = "B";
	  else
	    res = set;
	}
      }
      return res;
    }


    protected static String		get_type(NodeList l)
    {
      NamedNodeMap			attr;

      for (int i = 0; i < l.getLength(); i++)
      {
	Node elt = l.item(i);
	if (elt.getNodeType() == Node.ELEMENT_NODE &&
	    elt.getNodeName().equals("semiring"))
	{
	  attr = elt.getAttributes();
	  if (attr.getNamedItem("set") != null)
	    if (attr.getNamedItem("set").getNodeValue()
		.equals("ratSeries"))
	      return "Realtime transducer";
	}
	if (elt.getNodeType() == Node.ELEMENT_NODE &&
	    elt.getNodeName().equals("monoid"))
	{
	  attr = elt.getAttributes();
	  if (attr.getNamedItem("type") != null)
	    if (attr.getNamedItem("type").getNodeValue()
		.equals("product"))
	      return "FMP transducer";
	}
      }
      return "Automaton";
    }

    protected void			get_alphabets(NodeList l)
    {
      NamedNodeMap			attr;

      Debug.echo("get_alphabets!");
      for (int i = 0; i < l.getLength(); i++)
      {
	Node elt = l.item(i);
	boolean second_alphabet = false;

	if (elt.getNodeType() == Node.ELEMENT_NODE &&
	    elt.getNodeName().equals("monoid"))
	{
	  NodeList generators = elt.getChildNodes();
	  String alphabet = new String();

	  // Get generators value
	  for (int j = 0; j < generators.getLength(); j++)
	  {
	    Node gen = generators.item(j);

	    if (gen.getNodeType() == Node.ELEMENT_NODE &&
		gen.getNodeName().equals("generator"))
	    {
	      attr = gen.getAttributes();
	      alphabet += attr.getNamedItem("value").getNodeValue();
	    }
	  }
	  // Record alphabet
	  if (second_alphabet)
	    automaton_structure_.put(VJToolKitConstants.XML_ALPHA_OUT, alphabet);
	  else
	  {
	    automaton_structure_.put(VJToolKitConstants.XML_ALPHA_IN, alphabet);
	    second_alphabet = true;
	  }
	}
      }
    }

    // Set automaton algebraic structure to values retrieved in XML <type> tag.
    public void				set_automaton_structure(NodeList l)
    {
      Debug.echo("set_automaton_structure");
      automaton_structure_.put(VJToolKitConstants.XML_WEIGHT,
			       get_weight(l));
      automaton_structure_.put(VJToolKitConstants.XML_AUTOMATON_TYPE,
			       get_type(l));

      /// FIXME: Handle alphabets !
      get_alphabets(l);
    }

    // Set automaton algebraic structure to values retrieved in XML <type> tag.
    public void				set_automaton_structure(Map m)
    {
      automaton_structure_ = m;
    }


    // XML <geometry> tag related stuff, when anchored to <automaton> tag.
    //
    protected Map			automaton_geometry_;


    // Accessor.
    public Map				automaton_geometry_get()
    {
      return automaton_geometry_;
    }

    public void				automaton_geometry_set(Map geom)
    {
      automaton_geometry_ = geom;
    }


    // Useful graph manipulation functions.
    //
    // Get states connected to given state.
    // in:
    //	VJGraph
    //  State
    // out:
    //  State[]
    public static State[]		get_connected_states(VJGraph graph,
							     State state)
    {
      Vector res = new Vector();
      DefaultGraphModel model = (DefaultGraphModel) graph.getModel();
      Set edges = model.getEdges(model, new Object[]{state});
      Iterator iter = edges.iterator();

      while (iter.hasNext())
      {
	Object obj = iter.next();
	if (!(obj instanceof Transition))
	  continue;
	State src = ((Transition) obj).src_get();
	State dst = ((Transition) obj).dst_get();
	if (src != state && ! res.contains(src) && src != null)
	  res.add(src);
	if (dst != state && ! res.contains(dst) && dst != null)
	  res.add(dst);
      }
      return (State[]) res.toArray(new State[]{});
    }


    // Get all states.
    // in:
    //	VJGraph
    // out:
    //  State[]
    public static State[]		get_states(VJGraph graph)
    {
      Vector res = new Vector();
      Object objects[] = DefaultGraphModel.getAll(graph.getModel());

      for (int i = 0; i < objects.length; ++i)
      {
	Object obj = objects[i];
	if (obj instanceof State)
	  res.add(obj);
      }
      return (State[]) res.toArray(new State[]{});
    }


    // Get all initial states.
    // in:
    //	VJGraph
    // out:
    //  State[]
    public static State[]		get_initial_states(VJGraph graph)
    {
      return get_in_fin_states(graph, true);
    }


    // Get all final states.
    // in:
    //	VJGraph
    // out:
    //  State[]
    public static State[]		get_final_states(VJGraph graph)
    {
      return get_in_fin_states(graph, false);
    }


    // Sub-routine for get_initial_states and get_final_states.
    protected static State[]		get_in_fin_states(VJGraph graph,
							  boolean is_initial)
    {
      Vector res = new Vector();
      Object objects[] = DefaultGraphModel.getAll(graph.getModel());

      for (int i = 0; i < objects.length; ++i)
      {
	Object obj = objects[i];
	if (obj instanceof State)
	{
	  if (is_initial)
	  {
	    if (((State) obj).is_initial())
	      res.add(obj);
	  }
	  else
	    if (((State) obj).is_final())
	      res.add(obj);

	}
      }
      return (State[]) res.toArray(new State[]{});
    }


    // Get all transitions.
    // in:
    //	VJGraph
    // out:
    //  Transition[]
    public static Transition[]		get_transitions(VJGraph graph)
    {
      Vector res = new Vector();
      DefaultGraphModel model = (DefaultGraphModel) graph.getModel();
      Set objects =  model.getEdges(model, VJGraph.get_states(graph));
      Iterator iter = objects.iterator();

      while (iter.hasNext())
      {
	Object obj = iter.next();
	if (obj instanceof Transition)
	  res.add(obj);
      }
      return (Transition[]) res.toArray(new Transition[]{});
    }


    // Get loops connected to a given state.
    // in:
    //	VJGraph
    //  State
    // out:
    //  Transition[]
    public static Transition[]		get_loops(VJGraph graph,
						  State state)
    {
      Vector res = new Vector();
      DefaultGraphModel model = (DefaultGraphModel) graph.getModel();
      Set edges = model.getEdges(model, new Object[]{state});
      Iterator iter = edges.iterator();

      while (iter.hasNext())
      {
	Object obj = iter.next();
	if (!(obj instanceof Transition))
	  continue;
	State src = ((Transition) obj).src_get();
	State dst = ((Transition) obj).dst_get();
	if (src == dst)
	  res.add(obj);
      }
      return (Transition[]) res.toArray(new Transition[]{});
    }


    public static Object[]		get_selection(VJGraph graph)
    {
      return graph.getSelectionCells();
    }

    public Point2D			get_origin()
    {
      Point2D p = new Point2D.Double(getWidth() / 2, getHeight() / 2);

      if (p.getX() == 0 && p.getY() == 0)
	p.setLocation(VJToolKitConstants.WIDTH_GRAPH / 2,
		      VJToolKitConstants.HEIGHT_GRAPH / 2);

      Debug.echo("Oriigniiiii:");
      Debug.echo(p.getX());
      Debug.echo(p.getY());
      return p;
    }

    public int viewport_x;
    public int viewport_y;
}
