
//XMLConverter.java for vgi in /home/loulou/lrde_proj/vgi/src/vgitoolkit

//Made by Louis-Noel Pouchet
//Login   <loulou@lrde.epita.fr>

//Started on  Thu Mar  3 22:47:26 2005 Louis-Noel Pouchet
//Last update Tue Oct 16 17:02:59 2007 Guillaume



package vaucanson.vgitoolkit;


//Apache - Xerces Parser
import org.apache.xerces.parsers.DOMParser;
//DOM
import org.w3c.dom.*;

import org.xml.sax.SAXException;
import java.io.IOException;

//SAX
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.FactoryConfigurationError;
import javax.xml.parsers.ParserConfigurationException;


//Input / Output
import org.xml.sax.InputSource;
import java.io.FileReader;
import java.io.StringReader;
import java.io.FileNotFoundException;

import java.util.Iterator;
import java.util.Map;
import java.util.Hashtable;
import java.util.Set;
import java.util.Map.Entry;


//JGraph
import org.jgraph.graph.EdgeView;
import org.jgraph.graph.CellView;
import org.jgraph.graph.GraphModel;
import org.jgraph.graph.DefaultGraphModel;
import org.jgraph.graph.GraphLayoutCache;

//Others
import java.awt.geom.Rectangle2D;
import java.awt.geom.Point2D;


public class XMLConverter
{
	// Input: doc is an automaton.
	public static VJGraph	xml_to_graph(Document doc)
	{
		DefaultGraphModel model = new DefaultGraphModel();
		VJGraph ret = new VJGraph(model);

		if (doc == null)
			Debug.echo("Doc is null, WTF!!");

		ret.automaton_geometry_set(XMLReader.
				get_geometry((NodeList) doc));
		ret.set_automaton_structure(XMLReader.get_type(doc));
		ret.set_name((NodeList) doc);

		fill_graph(ret, doc);

		return ret;
	}

	public static Document	graph_to_xml(VJGraph graph)
	{
		String auto_type = (String) graph.automaton_structure_get()
		.get(VJToolKitConstants.XML_AUTOMATON_TYPE);
		Document type = create_document_type(graph);
		Document content = create_document_content(graph);

		return XMLWriter.build_automaton(type, content, auto_type,
				graph.name_get());
	}


	// graph_to_xml routines.


	/// FIXME: Use special function to store generators in model.
	public static void		create_letters(Document document,
			Element monoid,
			String a)
	{
		// Set letters.
		Element	letter;
		int i;

		if (a == null || a == "a, b" || a == "")
		{
			letter = document.createElement("generator");
			letter.setAttribute("value", "a");
			monoid.appendChild(letter);
			letter = document.createElement("generator");
			letter.setAttribute("value", "b");
			monoid.appendChild(letter);
		}
		else if (a == "a..z")
		{
			for (char c = 'a'; c <= 'z'; ++c)
			{
				letter = document.createElement("generator");
				letter.setAttribute("value", String.valueOf(c));
				monoid.appendChild(letter);
			}
		}
		else if (a == "0, 1")
		{
			letter = document.createElement("generator");
			letter.setAttribute("value", "0");
			monoid.appendChild(letter);
			letter = document.createElement("generator");
			letter.setAttribute("value", "1");
			monoid.appendChild(letter);
		}
		else if (a == "0..9")
		{
			for (char c = '0'; c <= '9'; ++c)
			{
				letter = document.createElement("generator");
				letter.setAttribute("value", String.valueOf(c));
				monoid.appendChild(letter);
			}
		}
		else if (a == "1..10")
		{
			for (char c = '1'; c <= '9'; ++c)
			{
				letter = document.createElement("generator");
				letter.setAttribute("value", String.valueOf(c));
				monoid.appendChild(letter);
			}
			letter = document.createElement("generator");
			letter.setAttribute("value", "10");
			monoid.appendChild(letter);
		}
		else if (a == "(a, 0), (b, 1)")
		{
			letter = document.createElement("generator");
			letter.setAttribute("value", "(a, 0)");
			monoid.appendChild(letter);
			letter = document.createElement("generator");
			letter.setAttribute("value", "(b, 1)");
			monoid.appendChild(letter);
		}
		else if (a == "(a, 0)..(k, 9)")
		{
			Debug.echo("Not implemented");
		}
		else if (a == "(a, a), (b, b)")
		{
			Debug.echo("Not implemented");
		}
		else if (a == "(a, a)..(z, z)")
		{
			Debug.echo("Not implemented");
		}
		else
		{
			Debug.echo("My way to write generators...");
			Debug.echo(a);

			for (int j = 0; j < a.length(); j++)
			{
				char c = a.charAt(j);

				letter = document.createElement("generator");
				letter.setAttribute("value", String.valueOf(c));
				monoid.appendChild(letter);
			}
		}
	}



	protected static void	create_free_monoid(Document doc,
			Element root,
			String alpha)
	{
		Element monoid = doc.createElement("monoid");
		monoid.setAttribute(VJToolKitConstants.XML_TYPE, "free");
		monoid.setAttribute(VJToolKitConstants.XML_GENERATORS, "letters");
		create_letters(doc, monoid, alpha);
		root.appendChild(monoid);
	}

	protected static void	create_product_monoid(Document doc,
			Element root,
			String alpha_in,
			String alpha_out)
	{
		Element monoid = doc.createElement("monoid");
		monoid.setAttribute(VJToolKitConstants.XML_TYPE, "product");
		create_free_monoid(doc, monoid, alpha_in);
		create_free_monoid(doc, monoid, alpha_out);
		root.appendChild(monoid);
	}

	protected static void	create_semiring(Document doc,
			Element root,
			String weight)
	{
		Element semiring = doc.createElement("semiring");
		if (weight.equals("(Z,max,+)"))
		{
			semiring.setAttribute(VJToolKitConstants.XML_OPERATIONS,
			"tropicalMax");
			semiring.setAttribute("set", "Z");
		}
		else if (weight.equals("(Z,min,+)"))
		{
			semiring.setAttribute(VJToolKitConstants.XML_OPERATIONS,
			"tropicalMin");
			semiring.setAttribute("set", "Z");
		}
		else
		{
			semiring.setAttribute("set", weight);
			semiring.setAttribute(VJToolKitConstants.XML_OPERATIONS,
					VJToolKitConstants.XML_OPERATIONS_DEFAULT);
		}
		root.appendChild(semiring);
	}


	protected static void	create_rat_semiring(Document doc,
			Element root,
			String alpha_out,
			String weight)
	{
		Element semiring = doc.createElement("semiring");
		semiring.setAttribute("set", "ratSeries");
		create_free_monoid(doc, semiring, alpha_out);
		create_semiring(doc, semiring, "B");
		root.appendChild(semiring);
	}



	public static Document	create_document_type(VJGraph graph)
	{
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();

		try
		{
			Map structure = graph.automaton_structure_get();
			String attrib;

			DocumentBuilder builder = factory.newDocumentBuilder();
			Document document = builder.newDocument();

			Element root = (Element)
			document.createElement("labelType");
			document.appendChild(root);

			String automaton_type = (String)
			structure.get(VJToolKitConstants.XML_AUTOMATON_TYPE);
			String weight = (String)
			structure.get(VJToolKitConstants.XML_WEIGHT);
			String alpha_in = (String)
			structure.get(VJToolKitConstants.XML_ALPHA_IN);
			String alpha_out = (String)
			structure.get(VJToolKitConstants.XML_ALPHA_OUT);

			if (automaton_type.equals("Automaton"))
			{
				create_free_monoid(document, root, alpha_in);
				create_semiring(document, root, weight);
			}
			else if (automaton_type.equals("FMP transducer"))
			{
				create_product_monoid(document, root,
						alpha_in, alpha_out);
				create_semiring(document, root, weight);
			}
			else
			{
				create_free_monoid(document, root, alpha_in);
				create_rat_semiring(document, root, alpha_out, weight);
			}

			return document;
		}
		catch (ParserConfigurationException pce)
		{
			pce.printStackTrace();
		}
		return null;
	}


	public static Document	create_document_content(VJGraph graph)
	{
		GraphLayoutCache glc = graph.getGraphLayoutCache();
		CellView objects[] = glc.getRoots();
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		int i;
		String type = (String) graph.automaton_structure_get()
		.get(VJToolKitConstants.XML_AUTOMATON_TYPE);

		try
		{
			DocumentBuilder builder = factory.newDocumentBuilder();
			Document document = builder.newDocument();

			Element root = (Element) document.createElement("content");
			document.appendChild(root);
			Element states = (Element) document.createElement("states");
			root.appendChild(states);

			Element transitions =
				(Element) document.createElement("transitions");
			root.appendChild(transitions);

			for (i = 0; i < objects.length; ++i)
			{
				Object obj = (objects[i]).getCell();
				if (obj instanceof State)
					document = add_state((State) obj,
							states,
							document,
							objects[i]);
				else if (obj instanceof Transition)
					document =
						add_transition((Transition) obj,
								transitions,
								document,
								objects[i],
								type);
			}
			for (i = 0; i < objects.length; ++i)
			{
				Object obj = (objects[i]).getCell();
				if (obj instanceof TerminalTransition)
				{
					TerminalTransition t =
						(TerminalTransition) obj;
					if (t.type_get() ==
						VJToolKitConstants.INITIAL_TRANSITION)
						document =
							add_initial_transition(t,
									transitions,
									document,
									objects[i],
									type);
				}
			}
			for (i = 0; i < objects.length; ++i)
			{
				Object obj = (objects[i]).getCell();

				if (obj instanceof TerminalTransition)
				{
					TerminalTransition t =
						(TerminalTransition) obj;
					if (t.type_get() !=
						VJToolKitConstants.INITIAL_TRANSITION)
						document =
							add_final_transition(t,
									transitions,
									document,
									objects[i],
									type);
				}
			}
			return document;
		}
		catch (ParserConfigurationException pce)
		{
			// Parser with specified options can't be built
			pce.printStackTrace();
		}
		return null;
	}

	protected static void	get_transition_label(Element t,
			String type,
			String label)
	{
		int i;

		if (label.length() == 0)
			return;
		if (type.equals("Automaton"))
			t.setAttribute("label", label);
		else if (type.equals("FMP transducer"))
		{
			String in = "";
			String out = "";
			for (i = 1; i < label.length() && label.charAt(i) != ','; ++i)
				in += label.charAt(i);
			for (++i; i < label.length() - 1; ++i)
				out += label.charAt(i);
			t.setAttribute("in", in);
			t.setAttribute("out", out);
		}
		else
		{
			String in = "";
			String out = "";
			for (i = 0; i < label.length() && label.charAt(i) != '|'; ++i)
				in += label.charAt(i);
			for (++i; i < label.length(); ++i)
				out += label.charAt(i);
			t.setAttribute("in", in);
			t.setAttribute("out", out);
		}
	}


	public static Document	add_transition(Transition trans,
			Node root,
			Document doc,
			Object obj,
			String type)
	{
		Element			xml_trans =
			(Element) doc.createElement("transition");

		/// FIXME: handle bad edge connections.
		xml_trans.setAttribute("src", trans.src_get().toString());
		xml_trans.setAttribute("dst", trans.dst_get().toString());
		String label = trans.get_label();
		get_transition_label(xml_trans, type, label);

		Element geom = (Element) doc.createElement("geometry");
		set_xml_geometry_trans(trans, geom, (EdgeView) obj);
		set_label_position(trans, geom, (EdgeView) obj);
		xml_trans.appendChild(geom);
		Element draw = (Element) doc.createElement("drawing");
		set_xml_drawing_trans(trans, draw);
		xml_trans.appendChild(draw);

		root.appendChild(xml_trans);
		return doc;
	}

	public static void		set_label_position(TransitionTemplate t,
			Element root,
			EdgeView edge)
	{
		int pos = GeometryProperties.getLabelPosition(t);
		Integer ipos = new Integer(pos);
		root.setAttribute(GeometryProperties.LABELPOSITION, ipos.toString());
	}


	public static Document	add_initial_transition(TerminalTransition
			trans,
			Node root,
			Document doc,
			Object obj,
			String type)
	{
		Element			xml_trans =
			(Element) doc.createElement("initial");

		/// FIXME: handle bad edge connections.
		xml_trans.setAttribute("state", trans.state_get().get_label());
		String label = trans.get_label();
		get_transition_label(xml_trans, type, label);

		Element geom = (Element) doc.createElement("geometry");
		set_xml_geometry_init_trans(trans, geom, (EdgeView) obj);
		set_label_position(trans, geom, (EdgeView) obj);
		xml_trans.appendChild(geom);

		Element draw = (Element) doc.createElement("drawing");
		set_xml_drawing_trans(trans, draw);
		xml_trans.appendChild(draw);

		root.appendChild(xml_trans);
		return doc;
	}


	public static Document	add_final_transition(TerminalTransition trans,
			Node root,
			Document doc,
			Object obj,
			String type)
	{
		Element			xml_trans =
			(Element) doc.createElement("final");

		/// FIXME: handle bad edge connections.
		xml_trans.setAttribute("state", trans.state_get().get_label());
		String label = trans.get_label();
		get_transition_label(xml_trans, type, label);

		Element geom = (Element) doc.createElement("geometry");
		set_xml_geometry_final_trans(trans, geom, (EdgeView) obj);
		set_label_position(trans, geom, (EdgeView) obj);
		xml_trans.appendChild(geom);
		Element draw = (Element) doc.createElement("drawing");
		set_xml_drawing_trans(trans, draw);
		xml_trans.appendChild(draw);

		root.appendChild(xml_trans);
		return doc;
	}


	public static Document	add_state(State state,
			Node root,
			Document doc,
			Object obj)
	{
		Element xml_state = (Element) doc.createElement("state");

		xml_state.setAttribute("name", state.name_get());
		xml_state.setAttribute("number",
				(new Integer(state.number_get())).toString());

		Element geom = (Element) doc.createElement("geometry");
		set_xml_geometry_state(state, geom, (CellView) obj);
		xml_state.appendChild(geom);

		root.appendChild(xml_state);
		return doc;
	}




	public static void		set_xml_geometry_state(State state,
			Element root,
			CellView cv)
	{
		int x = GeometryProperties.getX(state);
		int y = GeometryProperties.getY(state);
		root.setAttribute(GeometryProperties.X,
				(new Integer(x)).toString());
		root.setAttribute(GeometryProperties.Y,
				(new Integer(y)).toString());
	}

	public static void		set_xml_geometry_trans(Transition trans,
			Element root,
			EdgeView ev)
	{
		/// FIXME: Explore parameters in attribute map.

		// Label position is saved in set_label_position

		// Transition is a loop. Handle size and angle parameter.
		if (trans.dst_get() == trans.src_get())
		{
			int angle = GeometryProperties.getLoopDirection(trans);
			Integer ia = new Integer(angle);
			root.setAttribute(GeometryProperties.LOOPDIRECTION, ia.toString());
			return;
		}

		// Transition type: line, arcL, arcR, curve
		String trans_type = GeometryProperties.getTransitionType(trans);
		Debug.echo("transition type:");
		Debug.echo(trans_type);
		root.setAttribute(GeometryProperties.TRANSITIONTYPE,
				trans_type == null ?
						GeometryProperties.LINE :
							trans_type);


	}

	public static void		set_xml_geometry_init_trans(TerminalTransition trans,
			Element root,
			EdgeView ev)
	{

		// Transition angle
		int angle = GeometryProperties.getTransitionAngle(trans);
		Integer ia = new Integer(angle);
		root.setAttribute(GeometryProperties.TRANSITIONANGLE, ia.toString());
	}

	public static void		set_xml_geometry_final_trans
	(TerminalTransition trans, Element root, EdgeView ev)
	{
		// Transition angle
		int angle = GeometryProperties.getTransitionAngle(trans);
		Integer ia = new Integer(angle);
		root.setAttribute(GeometryProperties.TRANSITIONANGLE, ia.toString());
	}



	// Save drawing properties
	public static void		set_xml_drawing_trans(TransitionTemplate t, Element root)
	{
		int offset = DrawingProperties.getLabelOffset(t);
		Integer off = new Integer(offset);

		//root.setAttribute(DrawingProperties.LABELOFFSET, off.toString());
	}

	////////////////////////////////////////




	// xml_to_graph routines.
	public static void		fill_graph(VJGraph graph,
			Document content)
	{
		String type = (String) graph.automaton_structure_get()
		.get(VJToolKitConstants.XML_AUTOMATON_TYPE);

		NodeList l =
			((Element)(content.getElementsByTagName("states").item(0)))
			.getChildNodes();
		Map states_geometry = XMLReader.get_geometry(l);
		Map name_to_state = new Hashtable();
		fill_states(graph, l, name_to_state, states_geometry);

		l = ((Element)(content.getElementsByTagName("transitions").item(0)))
		.getChildNodes();
		Map transition_geometry = XMLReader.get_geometry(l);
		fill_transitions(graph, l, name_to_state, transition_geometry, type);
	}


	protected static void	fill_states(VJGraph graph,
			NodeList l,
			Map name_to_state,
			Map states_geometry)
	{
		int id = 0;
		GraphModel model = graph.getModel();
		NamedNodeMap attr;
		String name, label;
		int number;

		for (int i = 0; i < l.getLength(); i++)
		{
			Node elt = l.item(i);
			if (elt.getNodeType() == Node.ELEMENT_NODE &&
					elt.getNodeName().equals("state"))
			{
				id++;
				attr = elt.getAttributes();
				name = attr.getNamedItem("name").getNodeValue();

				// Both number and label are implied in DTD.
				number = attr.getNamedItem("number") == null ?
						id : new Integer(attr.getNamedItem("number")
								.getNodeValue()).intValue();
				label = attr.getNamedItem("label") == null ?
						name : attr.getNamedItem("label").getNodeValue();

				State state = new State(name,
						number,
						label,
						Tools.merge_map
						(states_geometry,
								XMLReader.get_geometry
								(elt.getChildNodes())),
								model,
								graph);
				name_to_state.put(name, state);
			}
		}
	}


	protected static String	build_label(NamedNodeMap attrib, String type)
	{
		String res;
		String label = "";
		String in = "1", out = "1";
		if (attrib.getNamedItem("label") != null)
			label = attrib.getNamedItem("label").getNodeValue();
		if (attrib.getNamedItem("in") != null)
			in = attrib.getNamedItem("in").getNodeValue();
		if (attrib.getNamedItem("out") != null)
			out = attrib.getNamedItem("out").getNodeValue();

		if (type.equals("Automaton"))
			return label;
		if (type.equals("FMP transducer"))
			return "(" + in + "," + out + ")";
		return in + "|" + out;
	}


	protected static void	fill_label_position(VJGraph graph,
			TransitionTemplate t,
			Map attr)
	{
		Point2D p;

		//         if (attr.get("labelPositionX") != null &&
		//                 attr.get("labelPositionY") != null)
		//         {
		//             Double x = new Double((String) attr.get("labelPositionX"));
		//             Double y = new Double((String) attr.get("labelPositionY"));
		//             p = new Point2D.Double(x.doubleValue(), y.doubleValue());
		//         }
		//         else
		//         {
		//             if (t instanceof Transition)
		//             {
		//                 Transition tt = (Transition) t;
		//                 if (tt.src_get() == tt.dst_get())
		//                     p = new Point2D.Double(2090, -14);
		//                 else
		//                     p = new Point2D.Double(600, -7);
		//             }
		//             else
		//                 p = new Point2D.Double(600, -7);
		//         }
		//         t.label_position_set(p);
	}


	protected static void	fill_transitions(VJGraph graph,
			NodeList l,
			Map name_to_state,
			Map transition_geometry,
			String type)
	{
		GraphModel model = graph.getModel();
		NamedNodeMap attr;
		String src, dst;

		for (int i = 0; i < l.getLength(); i++)
		{
			Node elt = l.item(i);
			if (elt.getNodeType() == Node.ELEMENT_NODE &&
					elt.getNodeName().equals("transition"))
			{
				attr = elt.getAttributes();

				src = attr.getNamedItem("src").getNodeValue();
				dst = attr.getNamedItem("dst").getNodeValue();

				Transition trans =
					new Transition(null,
							(State) name_to_state.get(src),
							(State) name_to_state.get(dst),
							build_label(attr, type),
							null,
							Tools.merge_map
							(transition_geometry,
									XMLReader.get_geometry
									(elt.getChildNodes())),
									model,
									graph);
				Map drawing = XMLReader.get_drawing(elt.getChildNodes());
				trans.drawing_set(drawing);
				fill_label_position(graph, trans, drawing);

			}
			if (elt.getNodeType() == Node.ELEMENT_NODE &&
					elt.getNodeName().equals("initial"))
			{
				attr = elt.getAttributes();
				String state =
					attr.getNamedItem("state").getNodeValue();
				State st = (State) name_to_state.get(state);
				st.set_initial(build_label(attr, type), null,
						XMLReader.get_geometry
						(elt.getChildNodes()));
				Map drawing = XMLReader.get_drawing(elt.getChildNodes());
				st.initial_transition_get().drawing_set(drawing);
				fill_label_position(graph,
						st.initial_transition_get(),
						drawing);
			}
			if (elt.getNodeType() == Node.ELEMENT_NODE &&
					elt.getNodeName().equals("final"))
			{
				attr = elt.getAttributes();
				String state =
					attr.getNamedItem("state").getNodeValue();
				State st = (State) name_to_state.get(state);
				st.set_final(build_label(attr, type), null,
						XMLReader.get_geometry
						(elt.getChildNodes()));
				Map drawing = XMLReader.get_drawing(elt.getChildNodes());
				st.final_transition_get().drawing_set(drawing);
				fill_label_position(graph,
						st.final_transition_get(),
						drawing);
			}
		}
	}


}
