
//XMLWriter.java for vgi in /home/loulou/lrde_proj/vgi/src/vgitoolkit

//Made by Louis-Noel Pouchet
//Login   <loulou@lrde.epita.fr>

//Started on  Sun Mar  6 20:51:21 2005 Louis-Noel Pouchet
//Last update Tue Oct 16 17:15:46 2007 Guillaume



package vaucanson.vgitoolkit;


import org.w3c.dom.*;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.StringWriter;

import org.apache.xml.serialize.XMLSerializer;
import org.apache.xml.serialize.OutputFormat;

public class XMLWriter
{
	public static void		graph_to_file(VJGraph graph, String filename)
	{
		try 
		{
			OutputFormat format = new OutputFormat();
			
			format.setLineSeparator("\n");
			format.setIndenting(true);
			
			XMLSerializer serializer = new XMLSerializer(format);
		    serializer.setOutputCharStream(
		      new java.io.FileWriter(filename));
		    serializer.serialize(XMLConverter.graph_to_xml(graph));
		}
		catch (FileNotFoundException e)
		{
			System.err.println(e);
		}
		catch (IOException e)
		{
			System.err.println(e);
		}
	}

	// Shortcut.
	public static String	graph_to_string(VJGraph graph)
	{
		StringWriter out = new StringWriter();
		try
		{
			XMLSerializer serializer = new XMLSerializer();
			
			serializer.setOutputCharStream(out);
			serializer.serialize(XMLConverter.graph_to_xml(graph));
		}
		catch (IOException e)
		{
			System.err.println(e);
		}
		return out.toString();
	}


	// Shortcut.
	public static String	graphs_to_string(VJGraph g1,
			VJGraph g2)
	{
		return print_document(build_session(XMLConverter.graph_to_xml(g1),
				XMLConverter.graph_to_xml(g2)));
	}


	public static Document	build_automaton(Document type, 
			Document content,
			String auto_type,
			String name)
	{
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();

		try
		{
			DocumentBuilder builder = factory.newDocumentBuilder();
			Document document = builder.newDocument();
			Element root;
			if (auto_type.equals("Automaton"))
				root = (Element) document.createElement("automaton");
			else
				root = (Element) document.createElement("transducer");
			root.setAttribute("xmlns", VJToolKitConstants.VAUCANSON_URL);
			root.setAttribute("name", name);
			Element type_elt = 
				(Element) document.importNode(type.getDocumentElement(), 
						true);
			Element content_elt = 
				(Element) document.importNode(content.getDocumentElement(),
						true);
			root.appendChild(type_elt);
			root.appendChild(content_elt);
			document.appendChild(root);
			return document;
		}
		catch (ParserConfigurationException pce)
		{
			// Parser with specified options can't be built
			pce.printStackTrace();
		}
		return null;
	}


	public static Document	build_session(Document automaton1, 
			Document automaton2)
	{
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();

		try
		{
			DocumentBuilder builder = factory.newDocumentBuilder();
			Document document = builder.newDocument();
			Element root = (Element) document.createElement("session");
			// 		root.setAttribute("xmlns", VJToolKitConstants.VAUCANSON_URL);
			Element a1_elt = 
				(Element) document.importNode(automaton1
						.getDocumentElement(), true);
			Element a2_elt = 
				(Element) document.importNode(automaton2
						.getDocumentElement(), true);
			root.appendChild(a1_elt);
			root.appendChild(a2_elt);
			document.appendChild(root);
			return document;
		}
		catch (ParserConfigurationException pce)
		{
			// Parser with specified options can't be built
			pce.printStackTrace();
		}
		return null;
	}



	public static String	print_document(Document doc)
	{
		StringWriter out = new StringWriter();
		try
		{
			OutputFormat format = new OutputFormat();
			
			format.setLineSeparator("\n");
			format.setIndenting(true);
			
			XMLSerializer serializer = new XMLSerializer(format);
			
			serializer.setOutputCharStream(out);
			serializer.serialize(doc);
		}
		catch (IOException e)
		{
			System.err.println(e);
		}
		return out.toString();
	}

//	protected static String	print_node(Node node, String indent)  
//	{
//		String res = "";
//		boolean to_print = true;
//
//		switch (node.getNodeType()) 
//		{
//		case Node.DOCUMENT_NODE:
//			// FIXME: This should absolutely not be hardcoded.
//			//res += "<?xml version=\"1.0\" encoding=\"utf-16\" standalone=\"no\" ?>\n";
//			// recurse on each child
//			NodeList nodes = node.getChildNodes();
//			if (nodes != null) 
//			{
//				for (int i=0; i<nodes.getLength(); i++)
//					res += print_node(nodes.item(i), "");
//			}
//			break;
//
//		case Node.ELEMENT_NODE:
//			String name = node.getNodeName();
//			res += indent + "<" + name;
//			NamedNodeMap attributes = node.getAttributes();
//			for (int i=0; i<attributes.getLength(); i++) 
//			{
//				Node current = attributes.item(i);
//				res += " " + current.getNodeName() +
//				"=\"" + current.getNodeValue() +
//				"\"";
//			}
//			if (name == "geometry" || name == "generator" ||
//				name == "semiring")
//			{
//				res += "/>\n";
//				to_print = false;
//			}
//			else
//				res += ">\n";
//
//			// recurse on each child
//			NodeList children = node.getChildNodes();
//			if (children != null) 
//				for (int i=0; i<children.getLength(); i++) 
//					res += print_node(children.item(i), indent + "  ");
//
//			if (to_print)
//				res += indent + "</" + name + ">\n";
//			break;
//
//		case Node.TEXT_NODE:
//			res += node.getNodeValue();
//			break;
//		}
//		return res;
//	}

}
